extension SBSDKDocumentScannerViewController {
  func withFinderCopy() {
    let config = SBSDKBaseScannerViewFinderConfiguration()
    config.isViewFinderEnabled = viewFinderConfiguration.isViewFinderEnabled
    config.lineColor = viewFinderConfiguration.lineColor
    config.lineWidth = viewFinderConfiguration.lineWidth
    config.backgroundColor = viewFinderConfiguration.backgroundColor
    config.lineCornerRadius = viewFinderConfiguration.lineCornerRadius
    config.minimumInset = viewFinderConfiguration.minimumInset
    config.aspectRatio = viewFinderConfiguration.aspectRatio
    viewFinderConfiguration = config
  }
  
  func withGeneralConfigCopy() {
    let config = SBSDKBaseScannerGeneralConfiguration()
    config.photoQualityPrioritization = generalConfiguration.photoQualityPrioritization
    config.videoContentMode = generalConfiguration.videoContentMode
    generalConfiguration = config
  }
    
  func withAutoSnapConfigCopy() {
      let config = SBSDKDocumentScannerPolygonConfiguration()
      config.enabled = autoSnapProgressPolygonConfiguration.enabled
      config.strokeWidth = autoSnapProgressPolygonConfiguration.strokeWidth
      config.strokeColor = autoSnapProgressPolygonConfiguration.strokeColor
      autoSnapProgressPolygonConfiguration = config
    }
}

extension UIColor {
    var transparentAlpha: UIColor {
        var alpha: CGFloat = 0
        getRed(nil, green: nil, blue: nil, alpha: &alpha)
        return withAlphaComponent(alpha * 0.2)
    }
}

struct DocumentScannerDefaultValues {
  let acceptedAngleScore = 75.0
  let acceptedBrightnessThreshold = 50
  let acceptedSizeScore = 80.0
  let autoSnappingEnabled = SBSDKAutoSnappingMode.enabled
  let autoSnappingSensitivity: Float = 0.66
  let autoSnappingDelay = 1.0
  let ignoreBadAspectRatio = false
  let cameraModule = SBSDKCameraDevice.defaultBackFacingCamera
  let cameraPreviewMode = SBSDKVideoContentMode.fillIn
  let photoQualityPrioritization = SBSDKCapturePhotoQualityPrioritization.balanced
  let finderEnabled = false
  let finderLineColor = UIColor.fromRgbAHex("#FFFFFF")
  let finderOverlayColor = UIColor.fromRgbAHex("#000000CC")
  let finderLineWidth: Float = 2.0
  let finderCornerRadius = 10.0
  let finderMinimumPadding = UIEdgeInsets(top: 20.0, left: 20.0, bottom: 20.0, right: 20.0)
  let finderAspectRatio = SBSDKAspectRatio.init(width: 1.0, height: 1.4142)
  let polygonEnabled = true
  let polygonBackgroundColor = UIColor.fromRgbAHex("#FFFFFF").transparentAlpha
  let polygonBackgroundColorOK = UIColor.fromRgbAHex("#FFFFFF").transparentAlpha
  let polygonColor = UIColor.fromRgbAHex("#FF00004D")
  let polygonColorOK = UIColor.fromRgbAHex("#00CEA600")
  let polygonLineWidth = 3.0
  let polygonCornerRadius = 8.0
  let polygonAutoSnapProgressEnabled = true
  let polygonAutoSnapProgressColor = UIColor.fromRgbAHex("#87F2DE00")
  let polygonAutoSnapProgressLineWidth = 5.0
}



